/* VMS_SYS.C -- 30-OCT-1996 Uwe Zessin
   interface to (some) OpenVMS SYS$ routines

   02-NOV-1997 ...
   04-NOV-1997 ZE. -- convert functions to use PyArg_ParseTuple() instead of
			PyArg_Parse().
   29-NOV-1997 ZE. -- changes for Python V1.5B1
   12-DEC-1997 ZE. -- add interfaces to SYS$DELPRC, SYS$ASCUTC, SYS$BINUTC,
			SYS$GETUTC + SYS$NUMUTC. change vms_sys.canwak +
			vms_sys.forcex to return target pid.
   14-DEC-1997 ZE. -- add interface to SYS$TIMCON
   17-DEC-1997 ZE. -- change vms_sys.resume to return target pid.
   18-DEC-1997 ZE. -- change vms_sys.wake to return target pid.
   19-DEC-1997 ZE. -- add interface to SYS$SUSPND, change vms_sys.setpri to
			return target pid.
   21-DEC-1997 ZE. -- Still no prototype for sys$setddir in DEC C V5.6
			Provide a default one to stop compiler complaining.
   09-FEB-1998 ZE. -- add remaining DOC strings
   12-AUG-1998 ZE. -- conversion of 64-bit date+time 'hex-string'
			to Python long int
*/


#include "python.h"
#include "vmsdef.h"

#include <descrip.h>
#include <rmsdef.h>		/* RMS$_name	 */
#include <ssdef.h>		/* SS$_name	 */
#include <starlet.h>		/* SYS$name, ... */

#ifdef __DECC
#ifndef __DECC_VER	/* don't know if this is defined in all versions ... */
#define __DECC_VER 0
#endif /* __DECC_VER */
#if __DECC_VER < 50600000 /* @@ might need to change for newer version */
#ifndef __unknown_params
#define __unknown_params ...
#endif
#pragma __nostandard	  /* non-ANSI-Standard feature */
int sys$setddir(__unknown_params);
#pragma __standard
#endif /* __DECC_VER < 50600000 */
#endif /* __DECC */
/* ------------------------------------------------------------------------- */

static PyObject *vms_sys_Error;

static char vms_sys_doc__ [] =
"This is an interface to some OpenVMS SYS$ routines.\n\
See the VMS_SYS documentation for details.";

/* ------------------------------------------------------------------------- */
/* function prototypes for external routines */

/* ------------------------------ */

/* input a binary quadword, output a hex string */
/* you MUST provide storage for 16 characters at *t_quadword_hex ! */
long vms__cvt_quad2hex (long* q_quadword_bin, char* t_quadword_hex);

/* ------------------------------ */

/* input a hex string, output a binary quadword */
/* you MUST provide 16 hex characters at *t_quadword_hex ! */
long vms__cvt_hex2quad (char* t_quadword_hex, long* q_quadword_bin);

/* ------------------------------ */

/* input a binary quadword - output a Python long int */
PyObject * vms__cvt_quad2pylong (long q_quadword[2]);

/* ------------------------------ */

/* input a Python long int - output a binary quadword */
long vms__cvt_pylong2quad (PyObject * r_longint, long * q_quadword);

/* ------------------------------ */

/* input a binary octaword, output a hex string */
/* you MUST provide storage for 32 characters at *t_octaword_hex ! */
long vms__cvt_oct2hex (long* o_octaword_bin, char* t_octaword_hex);

/* ------------------------------ */

/* input a hex string, output a binary octaword */
/* you MUST provide 32 hex characters at *t_octaword_hex ! */
long vms__cvt_hex2oct (char* t_octaword_hex, long* o_octaword_bin);

/* ------------------------------------------------------------------------- */
static char vms_sys_asctim__doc[] =
"timbuf = vms_sys.asctim (timadr [,cvtflg])\n\
Convert Binary Time to ASCII String.";

PyObject *
vms_sys_asctim (PyObject *self, PyObject *args)
{
	PyObject		* ar_timadr;
	long			  q_timadr[2];		/* binary system time */

#define S_TIMBUF 23
	char			t_timbuf[S_TIMBUF];	/* ASCII time */
	struct dsc$descriptor_s r_timbuf;
	short int		w_timbuf_len;

	long			l_cvtflg;	/* conversion flg, $ASCTIM */

	long			x_no_arg;

	long			l_status;

	/* argument 1 can be empty, 'None' or PyLongInt */
	l_cvtflg = 0;					/* default */
	if (PyArg_ParseTuple(args, ""))
	{
	  /* no arguments given, indicate current date + time */
	  /* to comply with SYS$ASCTIM */
	  x_no_arg = 1;
	}
	else
	{
	  PyErr_Clear();
	  /* allow PyLongInt or None --v , argument 2 is optional or integer */
	  if (!PyArg_ParseTuple(args, "O|l",
	       &ar_timadr, &l_cvtflg))
	  {
	    /* argument 1 is not:  1) empty,  2) 'None',  3) PyLongInt */
	    /* -> return with error */
	    return NULL;
	  }
	  else
	  {
	    x_no_arg = 0;
	    if (ar_timadr == Py_None)		/* argument is 'None' */
	    {
	      /* no arguments given, indicate current date + time */
	      /* to comply with SYS$ASCTIM */
	      x_no_arg = 1;	/* cvtflg can still be non-default ! */
	    }
            else
	    {
	      /* ar_timadr must be PyLongInt */
	      if (!PyLong_Check(ar_timadr))
	      {
		PyErr_SetString(PyExc_ValueError,
			"argument 1: must be long int");
		return NULL;
	      }

	      /* convert PyLongInt to binary quadword */
	      l_status = vms__cvt_pylong2quad (ar_timadr, &q_timadr[0]);
	      if (l_status == -1)
	      {
		/* error string has been set */
		return NULL;
	      }
	    }
	  }
	}

	/* q_timadr contains date+time to convert, l_cvtflg is set up */

	/* set up descriptor for SYS$ASCTIM */
	r_timbuf.dsc$w_length  = S_TIMBUF;
	r_timbuf.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_timbuf.dsc$b_class   = DSC$K_CLASS_S;
	r_timbuf.dsc$a_pointer = &t_timbuf[0];

	/* do conversion */
	if (x_no_arg)
	{
	    l_status = sys$asctim
		(&w_timbuf_len		/* [timlen] */
		,&r_timbuf		/* timbuf   */
		,0			/* [timadr] */ /* omitted */
		,l_cvtflg		/* [cvtflg] */
		);
	}
	else
	{
	    l_status = sys$asctim
		(&w_timbuf_len		/* [timlen] */
		,&r_timbuf		/* timbuf   */
		,&q_timadr		/* [timadr] */
		,l_cvtflg		/* [cvtflg] */
		);
	}

	if (l_status == SS$_NORMAL)
	{
	    /* no error during conversion - return ASCII time to Python */
	    return Py_BuildValue
		("s#", &t_timbuf[0], (int)w_timbuf_len);
	}

	/* conversion failed - error */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_asctim () */

/* ------------------------------------------------------------------------- */
static char vms_sys_asctoid__doc[] =
"id, attrib = vms_sys.asctoid (name)\n\
Translate Identifier Name to Identifier.";

PyObject *
vms_sys_asctoid (PyObject *self, PyObject *args)
{
	struct dsc$descriptor_s   r_name;
	char			* at_name;
	long			  l_name_len;

	long			  l_id;
	long			  l_attrib;
	long			  l_status;

	if (!PyArg_ParseTuple(args, "s#", &at_name, &l_name_len))
	{
	    return NULL;
	}

	/* set up string descriptor for SYS$ASCTOID */
	r_name.dsc$w_length  = strlen(at_name);
	r_name.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_name.dsc$b_class   = DSC$K_CLASS_S;
	r_name.dsc$a_pointer = at_name;

	l_status = sys$asctoid
		(&r_name		/* name     */
		,&l_id			/* [id]     */
		,&l_attrib		/* [attrib] */
		);

	if (l_status == SS$_NORMAL)
	{
	    return Py_BuildValue ("ii", l_id, l_attrib);
	}

	/* conversion failed - error */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_asctoid () */

/* ------------------------------------------------------------------------- */
static char vms_sys_ascutc__doc[] =
"Convert UTC to ASCII\n\
timbuf = vms_sys.ascutc (utcadr [,cvtflg]).";

PyObject *
vms_sys_ascutc (PyObject *self, PyObject *args)
{
	char		* at_utcadr;		/* hex string of octaword */
	long		  l_utcadr_len;
	long		  o_utcadr[4];		/* 128-bit utc_date_time */

#ifndef S_TIMBUF
#define S_TIMBUF 23
#endif
	char			t_timbuf[S_TIMBUF];	/* ASCII time */
	struct dsc$descriptor_s r_timbuf;
	short int		w_timbuf_len;

	long			l_cvtflg;	/* conversion flag, $ASCUTC */

	long			x_no_arg;

	long			l_status;

	/* argument 1 can be empty, 'None' or string */
	l_cvtflg = 0;					/* default */
	if (PyArg_ParseTuple(args, ""))
	{
	  /* no arguments given, indicate current date + time */
	  /* to comply with SYS$ASCUTC */
	  x_no_arg = 1;
	}
	else
	{
	  PyErr_Clear();
	  /* allow string or None -----v , argument 2 is optional or integer */
	  if (!PyArg_ParseTuple(args, "z#|l",
	       &at_utcadr, &l_utcadr_len, &l_cvtflg))
	  {
	    /* argument 1 is not:  1) empty,  2) 'None',  3) a string */
	    /* -> return with error */
	    return NULL;
	  }
	  else
	  {
	    x_no_arg = 0;
	    if (at_utcadr == NULL)		/* argument is 'None' */
	    {
	      /* no arguments given, indicate current date + time */
	      /* to comply with SYS$ASCUTC */
	      x_no_arg = 1;	/* cvtflg can still be non-default ! */
	    }

	    /* empty string ('') = skip [utcadr] argument */
	    if (l_utcadr_len == 0)
	    {
	      x_no_arg = 1;	/* cvtflg can still be non-default ! */
	    }
	    else
	    {
	      /* at_utcadr must be *exactly* 32 (hex) characters! */
	      if (l_utcadr_len != 32)
	      {
		PyErr_SetString(PyExc_ValueError,
			"argument 1: must be 32 hex characters");
		return NULL;
	      }

	      /* convert hex string to binary octaword */
	      l_status = vms__cvt_hex2oct (at_utcadr, &o_utcadr[0]);
	      if (l_status != SS$_NORMAL)
	      {
		PyErr_SetString(PyExc_ValueError,
			"argument 1: must be 32 hex characters");
		return NULL;
	      }
	    }
	  }
	}

	/* o_utcadr contains date+time to convert, l_cvtflg is set up */

	/* set up descriptor for SYS$ASCUTC */
	r_timbuf.dsc$w_length  = S_TIMBUF;
	r_timbuf.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_timbuf.dsc$b_class   = DSC$K_CLASS_S;
	r_timbuf.dsc$a_pointer = &t_timbuf[0];

	/* do conversion */
	if (x_no_arg)
	{
	    l_status = sys$ascutc
		(&w_timbuf_len		/* [timlen] */
		,&r_timbuf		/* timbuf   */
		,0			/* [utcadr] */ /* omitted */
		,l_cvtflg		/* [cvtflg] */
		);
	}
	else
	{
	    l_status = sys$ascutc
		(&w_timbuf_len		/* [timlen] */
		,&r_timbuf		/* timbuf   */
		,&o_utcadr		/* [utcadr] */
		,l_cvtflg		/* [cvtflg] */
		);
	}

	if (l_status == SS$_NORMAL)
	{
	    /* no error during conversion - return ASCII time to Python */
	    return Py_BuildValue
		("s#", &t_timbuf[0], (int)w_timbuf_len);
	}

	/* conversion failed - error */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_ascutc () */

/* ------------------------------------------------------------------------- */
static char vms_sys_bintim__doc[] =
"timadr = vms_sys.bintim (timbuf)\n\
Convert ASCII String to Binary Time.";

PyObject *
vms_sys_bintim (PyObject *self, PyObject *args)
{
	char			* at_timbuf;	/* ASCII time */
	struct dsc$descriptor_s r_timbuf;
	long			l_timbuf_len;

	long			q_timadr[2];	/* binary system time */

	long			l_status;

	if (!PyArg_ParseTuple(args, "s#", &at_timbuf, &l_timbuf_len))
	{
	    return NULL;
	}

	/* set up descriptor for SYS$BINTIM */
	r_timbuf.dsc$w_length  = l_timbuf_len;
	r_timbuf.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_timbuf.dsc$b_class   = DSC$K_CLASS_S;
	r_timbuf.dsc$a_pointer = at_timbuf;

	/* do conversion */
	l_status = sys$bintim
		(&r_timbuf		/* timbuf */
		,&q_timadr		/* timadr */
		);

	if (l_status == SS$_NORMAL)
	{
	    /* convert binary quadword to PyLongInt */
	    return vms__cvt_quad2pylong (&q_timadr[0]);
	}

	/* conversion failed - error */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_bintim () */

/* ------------------------------------------------------------------------- */
static char vms_sys_binutc__doc[] =
"utcadr = vms_sys.binutc (timbuf)\n\
Convert ASCII String to UTC Binary Time.";

PyObject *
vms_sys_binutc (PyObject *self, PyObject *args)
{
	char			* at_timbuf;	/* ASCII time */
	struct dsc$descriptor_s   r_timbuf;
	long			  l_timbuf_len;

	long			  o_utcadr[4];	/* utc_date_time */
	char			  t_utcadr[32];	/* hex string of octaword */

	long			  l_status;

	if (!PyArg_ParseTuple(args, "s#", &at_timbuf, &l_timbuf_len))
	{
	    return NULL;
	}

	/* set up descriptor for SYS$BINUTC */
	r_timbuf.dsc$w_length  = l_timbuf_len;
	r_timbuf.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_timbuf.dsc$b_class   = DSC$K_CLASS_S;
	r_timbuf.dsc$a_pointer = at_timbuf;

	/* do conversion */
	l_status = sys$binutc
		(&r_timbuf		/* timbuf */
		,&o_utcadr		/* utcadr */
		);

	if (l_status == SS$_NORMAL)
	{
	    /* convert binary octaword to hex string to return to Python */
	    l_status = vms__cvt_oct2hex (&o_utcadr[0], &t_utcadr[0]);

	    if (l_status == SS$_NORMAL)
	    {
		return Py_BuildValue ("s#", &t_utcadr[0], (int)32);
	    }
	    /* conversion failed - fall through to error */
	}

	/* conversion failed - error */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_binutc () */

/* ------------------------------------------------------------------------- */
static char vms_sys_canwak__doc[] =
"targpid = vms_sys.canwak ([pidadr] [,prcnam])\n\
Cancel Wakeup.";

PyObject *
vms_sys_canwak (PyObject *self, PyObject *args)
{
	long			  l_pidadr;
	long			* al_pidadr;
	PyObject		* ar_pidadr;

	char			* at_prcnam;
	struct dsc$descriptor_s   r_prcnam;
	struct dsc$descriptor_s	* ar_prcnam;
	long			  l_prcnam_len;

	long			  l_status;

	l_pidadr  = 0;
	/* argument 1 can be empty, 'None' or int */
	if (PyArg_ParseTuple(args, ""))
	{
	    l_pidadr  = 0;		/* skip arg 1 of SYS$CANWAK */
	    al_pidadr = &l_pidadr;	/*   but return PID	    */
	    ar_prcnam = 0;		/* skip arg 2 of SYS$CANWAK */
	    /* empty or None indicates CANWAK on current process */
	}
	else
	{
	  PyErr_Clear();
	  /* allow int or None --------v , argument 2 is opt, none or string */
	  if (!PyArg_ParseTuple(args, "O|z#",
		   &ar_pidadr, &at_prcnam, &l_prcnam_len))
	  {
	    return NULL;
	  }
	  else
	  {
	    /* argument 1 processing */
	    if (ar_pidadr == Py_None)	/* argument is 'None' */
	    {
	      /* turn 'None' into '0' because argument 2 might be present */
	      /*   and this routine has to return a PID */
	      l_pidadr  = 0;
	      al_pidadr = &l_pidadr;
	    }
	    else
	    {
	      if (!PyInt_Check(ar_pidadr))
	      {
		PyErr_SetString(PyExc_TypeError,
		    "argument 1: pidadr must be integer or None");
		return NULL;
	      }
	      else
	      {
		l_pidadr  = PyInt_AsLong(ar_pidadr);
		al_pidadr = &l_pidadr;
	      }
	    }

	    /* argument 2 processing */
	    if (at_prcnam == NULL)	/* argument is 'None' */
	    {
	      ar_prcnam = 0;		/* skip arg 2 of SYS$CANWAK */
	    }
	    else
	    {
	      /* set up descriptor */
	      r_prcnam.dsc$w_length  = (short)l_prcnam_len;
	      r_prcnam.dsc$b_dtype   = DSC$K_DTYPE_T;
	      r_prcnam.dsc$b_class   = DSC$K_CLASS_S;
	      r_prcnam.dsc$a_pointer = at_prcnam;
	      /* passed by reference */
	      ar_prcnam = &r_prcnam;
	    }
	  }
	}

	/* call system service */
	l_status = sys$canwak (al_pidadr, ar_prcnam);
	if (l_status == SS$_NORMAL)
	{
	    /* return PID of target process */
	    return Py_BuildValue ("l", l_pidadr);
	}

	/* call failed - error */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_canwak () */

/* ------------------------------------------------------------------------- */
static char vms_sys_delprc__doc[] =
"targpid = vms_sys.delprc ([pidadr] [,prcnam])\n\
Delete Process.";

PyObject *
vms_sys_delprc (PyObject *self, PyObject *args)
{
	long			  l_pidadr;
	long			* al_pidadr;
	PyObject		* ar_pidadr;

	char			* at_prcnam;
	struct dsc$descriptor_s   r_prcnam;
	struct dsc$descriptor_s * ar_prcnam;
	long			  l_prcnam_len;

	long			  l_status;

	/* argument 1 can be empty, 'None' or int */
	if (PyArg_ParseTuple(args, ""))
	{
	    al_pidadr = 0; /* skip arg 1 of SYS$DELPRC */
	    ar_prcnam = 0; /* skip arg 2 of SYS$DELPRC */
	    /* empty or None indicates deletion of current process */
	}
	else
	{
	  PyErr_Clear();
	  /* allow int or None --------v , argument 2 is opt, none or string */
	  if (!PyArg_ParseTuple(args, "O|z#",
		   &ar_pidadr, &at_prcnam, &l_prcnam_len))
	  {
	    return NULL;
	  }
	  else
	  {
	    /* argument 1 processing */
	    if (ar_pidadr == Py_None)	/* argument is 'None' */
	    {
	      /* turn 'None' into '0' because argument 2 might be present */
	      /*   and this routine has to return a PID */
	      l_pidadr  = 0;
	      al_pidadr = &l_pidadr;
	      /* empty or None without argument 2 indicates deletion of */
	      /* the current process */
	    }
	    else
	    {
	      if (!PyInt_Check(ar_pidadr))
	      {
		PyErr_SetString(PyExc_TypeError,
		    "argument 1: pidadr must be integer or None");
		return NULL;
	      }
	      else
	      {
		l_pidadr  = PyInt_AsLong(ar_pidadr);
		al_pidadr = &l_pidadr;
	      }
	    }

	    /* argument 2 processing */
	    if (at_prcnam == NULL)	/* argument is 'None' */
	    {
	      ar_prcnam = 0;		/* skip arg 2 of SYS$DELPRC */
	    }
	    else
	    {
	      /* set up descriptor */
	      r_prcnam.dsc$w_length  = (short)l_prcnam_len;
	      r_prcnam.dsc$b_dtype   = DSC$K_DTYPE_T;
	      r_prcnam.dsc$b_class   = DSC$K_CLASS_S;
	      r_prcnam.dsc$a_pointer = at_prcnam;
	      /* passed by reference */
	      ar_prcnam = &r_prcnam;
	    }
	  }
	}

	/* call system service */
	l_status = sys$delprc (al_pidadr, ar_prcnam);
	if (l_status == SS$_NORMAL)
	{
	    /* return PID of target process */
	    return Py_BuildValue ("l", l_pidadr);
	}

	/* call failed - error */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_delprc () */

/* ------------------------------------------------------------------------- */
static char vms_sys_forcex__doc[] =
"targpid = vms_sys.forcex ([pidadr] [,prcnam] [,code])\n\
Force Exit.";

PyObject *
vms_sys_forcex (PyObject *self, PyObject *args)
{
	long			  l_pidadr;
	long			* al_pidadr;
	PyObject		* ar_pidadr;

	char			* at_prcnam;
	struct dsc$descriptor_s   r_prcnam;
	struct dsc$descriptor_s	* ar_prcnam;
	long			  l_prcnam_len;

	long			  l_code;
	PyObject		* ar_code;

	long			  l_status;

	l_code = 0;		/* pre-set - 0 is used even if omitted */
	l_pidadr  = 0;
	/* argument 1 can be empty, 'None' or int */
	if (PyArg_ParseTuple(args, ""))
	{
	    al_pidadr = 0; /* skip arg 1 of SYS$FORCEX */
	    ar_prcnam = 0; /* skip arg 2 of SYS$FORCEX */
	    /* empty or None indicates forcex on current process */
	}
	else
	{
	  PyErr_Clear();
	  at_prcnam = NULL; /* set defaults for optional argument */
	  ar_code   = NULL; /* " */
	  /* allow int or None --------v , argument 2 is opt, none or string */
	  if (!PyArg_ParseTuple(args, "O|z#O",  /* arg 3 is opt, none or int */
		   &ar_pidadr, &at_prcnam, &l_prcnam_len, &ar_code))
	  {
	    return NULL;
	  }
	  else
	  {
	    /* argument 1 processing */
	    if (ar_pidadr == Py_None)	/* argument is 'None' */
	    {
	      /* turn 'None' into '0' because argument 2 might be present */
	      /*   and this routine has to return a PID */
	      l_pidadr  = 0;
	      al_pidadr = &l_pidadr;
	    }
	    else
	    {
	      if (!PyInt_Check(ar_pidadr))
	      {
		PyErr_SetString(PyExc_TypeError,
		    "argument 1: pidadr must be integer or None");
		return NULL;
	      }
	      else
	      {
		l_pidadr  = PyInt_AsLong(ar_pidadr);
		al_pidadr = &l_pidadr;
	      }
	    }

	    /* argument 2 processing */
	    if (at_prcnam == NULL)	/* argument is 'None' */
	    {
	      ar_prcnam = 0;		/* skip arg 2 of SYS$FORCEX */
	    }
	    else
	    {
	      /* set up descriptor */
	      r_prcnam.dsc$w_length  = (short)l_prcnam_len;
	      r_prcnam.dsc$b_dtype   = DSC$K_DTYPE_T;
	      r_prcnam.dsc$b_class   = DSC$K_CLASS_S;
	      r_prcnam.dsc$a_pointer = at_prcnam;
	      /* passed by reference */
	      ar_prcnam = &r_prcnam;
	    }

	    /* argument 3 processing */
	    if (ar_code == NULL)	/* argument omitted */
	    {
	      l_code = 0;		/* arg 3 = default */
	    }
	    else
	    {
	      if (ar_code == Py_None)	/* argument is 'None' */
	      {
		l_code = 0; /* arg 3 = default */
	      }
	      else
	      {
		if (!PyInt_Check(ar_code))
		{
		  PyErr_SetString(PyExc_TypeError,
		      "argument 3: code must be integer or None");
		  return NULL;
		}
		else
		{
		  l_code = PyInt_AsLong(ar_code);
		}
	      }
	    }
	  }
	}

	/* call system service */
	l_status = sys$forcex
		(al_pidadr		/* [pidadr] */
		,ar_prcnam		/* [prcnam] */
		,l_code			/* [code]   */
		);
	if (l_status == SS$_NORMAL)
	{
	    /* return PID of target process */
	    return Py_BuildValue ("l", l_pidadr);
	}

	/* call failed - error */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_forcex () */

/* ------------------------------------------------------------------------- */
static char vms_sys_gettim__doc[] =
"timadr = vms_sys.gettim ()\n\
Returns the current system time in a 64-bit format.";

PyObject *
vms_sys_gettim (PyObject *self, PyObject *args)
{
	long	q_timadr[2];		/* binary system time */
	long	l_status;

	/* no arguments taken */
	if (!PyArg_ParseTuple(args, ""))
	{
	    return NULL;
	}

	l_status = sys$gettim (&q_timadr[0]);	/* get current system time */

	if (l_status == SS$_NORMAL)
	{
	    /* convert binary quadword to PyLongInt */
	    return vms__cvt_quad2pylong (&q_timadr[0]);
	}

	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_gettim () */

/* ------------------------------------------------------------------------- */
static char vms_sys_getutc__doc[] =
"utcadr = vms_sys.gettim ()\n\
Get UTC Time.";

PyObject *
vms_sys_getutc (PyObject *self, PyObject *args)
{
	char	t_gettim_string[32];	/* 32 character hex quadword */
	long	o_timadr[4];		/* utc_date_time */
	long	l_status;

	/* no arguments taken */
	if (!PyArg_ParseTuple(args, ""))
	{
	    return NULL;
	}

	l_status = sys$getutc (&o_timadr[0]);	/* get current utc_date_time */

	if (l_status == SS$_NORMAL)
	{
	    /* convert binary octaword to hex string to return to Python */
	    l_status = vms__cvt_oct2hex (&o_timadr[0], &t_gettim_string[0]);

	    if (l_status == SS$_NORMAL)
	    {
		return Py_BuildValue
		    ("s#", &t_gettim_string[0], (int)32);
	    }
	    /* conversion failed - fall through to error */
	}

	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_getutc () */

/* ------------------------------------------------------------------------- */
static char vms_sys_hiber__doc[] =
"vms_sys.hiber ()\n\
Hibernate.";

PyObject *
vms_sys_hiber (PyObject *self, PyObject *args)
{
	long	l_status;

	/* no arguments taken */
	if (!PyArg_ParseTuple(args, ""))
	{
	    return NULL;
	}

	l_status = sys$hiber();		/* hibernate process */

	if (l_status == SS$_NORMAL)	/* should always return it */
	{
	    Py_INCREF(Py_None);
	    return Py_None;
	}

	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_hiber () */

/* ------------------------------------------------------------------------- */
static char vms_sys_numtim__doc[] =
"timbuf = vms_sys.numtim ([timadr])\n\
Convert Binary Time to Numeric Time.";

PyObject *
vms_sys_numtim (PyObject *self, PyObject *args)
{
	PyObject	* ar_timadr;		/* Python Long Int */
	long		  q_timadr[2];		/* binary system time */
	long		* aq_timadr;

	short int	  w_timbuf[7];
#define K_YEAR_SINCE_0         0
#define K_MONTH_OF_YEAR        1
#define K_DAY_OF_MONTH         2
#define K_HOUR_OF_DAY          3
#define K_MINUTE_OF_HOUR       4
#define K_SECOND_OF_MINUTE     5
#define K_HUNDREDTHS_OF_SECOND 6

	long		  l_status;

	ar_timadr = NULL;		/* set default if no argument */
	if (!PyArg_ParseTuple(args, "|O", &ar_timadr))
	{
	    return NULL;
	}

	if ((ar_timadr == NULL) || (ar_timadr == Py_None))
	{
	    /* no argument or argument 1 is 'None' */
	    /* indicate current date + time	   */
	    aq_timadr = 0;
	}
	else
	{
	    /* ar_timadr must be PyLongInt */
	    if (!PyLong_Check(ar_timadr))
	    {
		PyErr_SetString(PyExc_ValueError,
			"argument 1: must be long int");
		return NULL;
	    }

	    /* convert PyLongInt to binary quadword */
	    l_status = vms__cvt_pylong2quad (ar_timadr, &q_timadr[0]);
	    if (l_status == -1)
	    {
		/* error string has been set */
		return NULL;
	    }
	    aq_timadr = &q_timadr[0];
	}

	l_status = sys$numtim
		(&w_timbuf[0]		/* timbuf   */
		,aq_timadr		/* [timadr] */
		);
	if (l_status == SS$_NORMAL)
	{
	    /* return tuple to Python */
	    return Py_BuildValue
		("(iiiiiii)"
		,(int)w_timbuf[K_YEAR_SINCE_0]
		,(int)w_timbuf[K_MONTH_OF_YEAR]
		,(int)w_timbuf[K_DAY_OF_MONTH]
		,(int)w_timbuf[K_HOUR_OF_DAY]
		,(int)w_timbuf[K_MINUTE_OF_HOUR]
		,(int)w_timbuf[K_SECOND_OF_MINUTE]
		,(int)w_timbuf[K_HUNDREDTHS_OF_SECOND]
		);
	}

	/* failed, error */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_numtim () */

/* ------------------------------------------------------------------------- */
static char vms_sys_numutc__doc[] =
"timbuf = vms_sys.numtim ([utcadr])\n\
Convert UTC Time to Numeric Components.";

PyObject *
vms_sys_numutc (PyObject *self, PyObject *args)
{
	char		* at_utcadr;		/* hex string of octaword */
	long		  l_utcadr_len;
	long		  o_utcadr[4];		/* utc_date_time */
	long		* ao_utcadr;

	struct	numutc_xr_timbuf {
		unsigned short int	w_year_since_0;
		unsigned short int	w_month_of_year;
		unsigned short int	w_day_of_month;
		unsigned short int	w_hour_of_day;
		unsigned short int	w_minute_of_hour;
		unsigned short int	w_second_of_minute;
		unsigned short int	w_hundredths_of_second;
		short int		w_inacc_days;	/* [1] */
		short int		w_inacc_hours;
		short int		w_inacc_minutes;
		short int		w_inacc_seconds;
		short int		w_inacc_hundredths_of_second;
		short int		w_tdf_in_minutes;
	} numutc_r_timbuf;

	/* [1] - signed variable, */
	/* because a negative value indicates infinite inaccuracy */

	long		l_status;

	at_utcadr = NULL;		/* set default if no argument */
	if (!PyArg_ParseTuple(args, "|z#", &at_utcadr, &l_utcadr_len))
	{
	    return NULL;
	}

	if (at_utcadr == NULL)
	{
	    /* no argument or argument 1 is 'None' */
	    /* indicate current date + time	   */
	    ao_utcadr = 0;
	}
	else
	{
	    /* at_utcadr must be *exactly* 32 (hex) characters! */
	    if (l_utcadr_len != 32)
	    {
		PyErr_SetString(PyExc_ValueError,
		    "argument 1 must be 32 hex characters");
		return NULL;
	    }

	    /* convert hex string to binary quadword */
	    l_status = vms__cvt_hex2oct (at_utcadr, &o_utcadr[0]);
	    if (l_status != SS$_NORMAL)
	    {
		PyErr_SetString(PyExc_ValueError,
		    "argument 1 must be 32 hex characters");
		return NULL;
	    }
	    ao_utcadr = &o_utcadr[0];
	}

	l_status = sys$numutc
		(&numutc_r_timbuf	/* timbuf   */
		,ao_utcadr		/* [utcadr] */
		);
	if (l_status == SS$_NORMAL)
	{
	    /* return tuple to Python */
	    return Py_BuildValue
		("(iiiiiiiiiiiii)"
		,(int)numutc_r_timbuf.w_year_since_0
		,(int)numutc_r_timbuf.w_month_of_year
		,(int)numutc_r_timbuf.w_day_of_month
		,(int)numutc_r_timbuf.w_hour_of_day
		,(int)numutc_r_timbuf.w_minute_of_hour
		,(int)numutc_r_timbuf.w_second_of_minute
		,(int)numutc_r_timbuf.w_hundredths_of_second
		,(int)numutc_r_timbuf.w_inacc_days
		,(int)numutc_r_timbuf.w_inacc_hours
		,(int)numutc_r_timbuf.w_inacc_minutes
		,(int)numutc_r_timbuf.w_inacc_seconds
		,(int)numutc_r_timbuf.w_inacc_hundredths_of_second
		,(int)numutc_r_timbuf.w_tdf_in_minutes
		);
	}

	/* failed, error */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_numutc () */

/* ------------------------------------------------------------------------- */
static char vms_sys_purgws__doc[] =
"vms_sys.purgws (start-addr, end-addr)\n\
Purge Working Set.";

PyObject *
vms_sys_purgws (PyObject *self, PyObject *args)
{
	long	l_adr_low;
	long	l_adr_high;
	long	q_inadr[2];
	long	l_status;

	if (!PyArg_ParseTuple(args, "ll", &l_adr_low, &l_adr_high))
	{
	    return NULL;
	}

	q_inadr[0] = l_adr_low;
	q_inadr[1] = l_adr_high;

	l_status = sys$purgws (&q_inadr[0]);
	if (l_status == SS$_NORMAL)
	{
	    Py_INCREF(Py_None);
	    return Py_None;
	}

	/* failed, error */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_purgws() */

/* ------------------------------------------------------------------------- */
static char vms_sys_resume__doc[] =
"targpid = vms_sys.resume ([pidadr] [,prcnam])\n\
Resume Process.";

PyObject *
vms_sys_resume (PyObject *self, PyObject *args)
{
	long			  l_pidadr;
	long			* al_pidadr;
	PyObject		* ar_pidadr;

	char			* at_prcnam;
	struct dsc$descriptor_s   r_prcnam;
	struct dsc$descriptor_s	* ar_prcnam;
	long			  l_prcnam_len;

	long			  l_status;

	/* argument 1 can be empty, 'None' or int */
	if (PyArg_ParseTuple(args, ""))
	{
	    l_pidadr  = 0;		/* skip arg 1 of SYS$RESUME */
	    al_pidadr = &l_pidadr;	/*   but return PID	    */
	    ar_prcnam = 0;		/* skip arg 2 of SYS$RESUME */
	}
	else
	{
	  PyErr_Clear();
	  at_prcnam = NULL; /* set defaults for optional argument */
	  /* allow int or None --------v , argument 2 is opt, none or string */
	  if (!PyArg_ParseTuple(args, "O|z#",
		   &ar_pidadr, &at_prcnam, &l_prcnam_len))
	  {
	    return NULL;
	  }
	  else
	  {
	    /* argument 1 processing */
	    if (ar_pidadr == Py_None)	/* argument is 'None' */
	    {
	      /* turn 'None' into '0' because argument 2 might be present */
	      /*   and this routine has to return a PID */
	      l_pidadr  = 0;
	      al_pidadr = &l_pidadr;
	    }
	    else
	    {
	      if (!PyInt_Check(ar_pidadr))
	      {
		PyErr_SetString(PyExc_TypeError,
		    "argument 1: pidadr must be integer or None");
		return NULL;
	      }
	      else
	      {
		l_pidadr  = PyInt_AsLong(ar_pidadr);
		al_pidadr = &l_pidadr;
	      }
	    }

	    /* argument 2 processing */
	    if (at_prcnam == NULL)	/* argument is 'None' */
	    {
	      ar_prcnam = 0;		/* skip arg 2 of SYS$CANWAK */
	    }
	    else
	    {
	      /* set up descriptor */
	      r_prcnam.dsc$w_length  = (short)l_prcnam_len;
	      r_prcnam.dsc$b_dtype   = DSC$K_DTYPE_T;
	      r_prcnam.dsc$b_class   = DSC$K_CLASS_S;
	      r_prcnam.dsc$a_pointer = at_prcnam;
	      /* passed by reference */
	      ar_prcnam = &r_prcnam;
	    }
	  }
	}

	/* call system service */
	l_status = sys$resume (al_pidadr, ar_prcnam);
	if (l_status == SS$_NORMAL)
	{
	    /* return PID of target process */
	    return Py_BuildValue ("l", l_pidadr);
	}

	/* failed - error */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_resume () */

/* ------------------------------------------------------------------------- */
static char vms_sys_schdwk__doc[] =
"targpid = vms_sys.schdwk ([pidadr], [prcnam], daytim [,reptim])\n\
Schedule Wakeup.";

PyObject *
vms_sys_schdwk (PyObject *self, PyObject *args)
{
	long			  l_pidadr;
	long			* al_pidadr;
	PyObject		* ar_pidadr;

	char			* at_prcnam;
	struct dsc$descriptor_s   r_prcnam;
	struct dsc$descriptor_s	* ar_prcnam;
	long			  l_prcnam_len;

	PyObject		* ar_daytim;
	long			  q_daytim[2];

	PyObject		* ar_reptim;
	long			  q_reptim[2];
	long			* aq_reptim;

	long			  l_status;

	at_prcnam    = NULL;	/* should be set if 'None' was given, anyway */
	ar_daytim    = NULL;
	ar_reptim    = NULL;	/* set default to allow ommitted argument */

	if (!PyArg_ParseTuple(args, "Oz#O|O",
		&ar_pidadr, &at_prcnam, &l_prcnam_len, &ar_daytim, &ar_reptim))
	{
	    return NULL;
	}

	/* argument 1 processing */
	if (ar_pidadr == Py_None)	/* argument is 'None' */
	{
	    l_pidadr = 0;		/* skip arg 1 of SYS$CANWAK */
	}
	else
	{
	    if (!PyInt_Check(ar_pidadr))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 1: pidadr must be integer or None");
		return NULL;
	    }
	    else
	    {
		l_pidadr  = PyInt_AsLong(ar_pidadr);
	    }
	}
	al_pidadr = &l_pidadr;	/* always return PID */

	/* argument 2 processing */
	if (at_prcnam == NULL)	/* argument is 'None' */
	{
	    ar_prcnam = 0;	/* skip arg 2 of SYS$SCHDWK */
	}
	else
	{
	    /* set up descriptor */
	    r_prcnam.dsc$w_length  = (short)l_prcnam_len;
	    r_prcnam.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_prcnam.dsc$b_class   = DSC$K_CLASS_S;
	    r_prcnam.dsc$a_pointer = at_prcnam;
	    /* passed by reference */
	    ar_prcnam = &r_prcnam;
	}

	/* argument 3 processing */
	if (!PyLong_Check(ar_daytim))
	{
	    PyErr_SetString(PyExc_ValueError,
		"argument 3: must be long int");
	    return NULL;
	}

	/* convert PyLongInt to binary quadword */
	l_status = vms__cvt_pylong2quad (ar_daytim, &q_daytim[0]);
	if (l_status == -1)
	{
	    /* error string has been set */
	    return NULL;
	}

	/* argument 4 processing */
	if ((ar_reptim == NULL)	|| (ar_reptim == Py_None)) /* arg is 'None' */
	{
	    aq_reptim = 0;		/* argument omitted */
	}
	else
	{
	    if (!PyLong_Check(ar_reptim))
	    {
		PyErr_SetString(PyExc_ValueError,
		    "argument 4: must be long int");
		return NULL;
	    }

	    l_status = vms__cvt_pylong2quad (ar_reptim, &q_reptim[0]);
	    if (l_status == -1)
	    {
		/* error string has been set */
		return NULL;
	    }
	    aq_reptim = &q_reptim[0];
	}

	/* call system service */
	l_status = sys$schdwk
		(al_pidadr		/* [pidadr] */
		,ar_prcnam		/* [prcnam] */
		,&q_daytim[0]		/* daytim   */
		,aq_reptim		/* [reptim] */
		);
	if (l_status == SS$_NORMAL)
	{
	    /* return PID of target process */
	    return Py_BuildValue ("l", l_pidadr);
	}

	/* failed - error */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_schdwk () */

/* ------------------------------------------------------------------------- */
/* Allows you to read and change the default directory string for
   the process.

   Note that the device name is stored in the logical name SYS$DISK ! */

static char vms_sys_setddir__doc[] =
"cur-dir = vms_sys.setddir ([new-dir])\n\
Set Default Directory.";

PyObject *
vms_sys_setddir (PyObject *self, PyObject *args)
{
	char			* at_newdir;
	struct dsc$descriptor_s   r_newdir;
	struct dsc$descriptor_s	* ar_newdir;
	long			  l_newdir_len;

	char			  t_curdir[256];
	struct dsc$descriptor_s   r_curdir;
	long			  l_curdir_len;
	short int		  w_curdir_len;

	long			  l_status;

	at_newdir = NULL;	/* provide a default if no argument is given */
	if (!PyArg_ParseTuple(args, "|z#", &at_newdir, &l_newdir_len))
	{
	    return NULL;
	}

	if (at_newdir == NULL)
	{
	    ar_newdir = 0;	/* don't set new directory */
	}
	else
	{
	    /* set up descriptor for new directory */
	    r_newdir.dsc$w_length  = (short)l_newdir_len;
	    r_newdir.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_newdir.dsc$b_class   = DSC$K_CLASS_S;
	    r_newdir.dsc$a_pointer = at_newdir;
	    /* passed by reference */
	    ar_newdir = &r_newdir;
	}

	/* set up descriptor for current directory */
	r_curdir.dsc$w_length  = sizeof(t_curdir);
	r_curdir.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_curdir.dsc$b_class   = DSC$K_CLASS_S;
	r_curdir.dsc$a_pointer = &t_curdir[0];

	w_curdir_len = 0;

	/* call system service */
	l_status = sys$setddir
		(ar_newdir		/* [new-dir-addr] */
		,&w_curdir_len		/* [length-addr]  */
		,&r_curdir		/* [cur-dir-addr] */
		);
	if (l_status == RMS$_NORMAL)
	{
	    l_curdir_len = w_curdir_len;
	    /* return ALWAYS the current directory */
	    return Py_BuildValue ("s#", &t_curdir, l_curdir_len);
	}

	/* failed - error */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_setddir () */

/* ------------------------------------------------------------------------- */
static char vms_sys_setpri__doc[] =
"targpid, previous_priority, previous_policy = vms_sys.setpri (\
[pid], [prcnam], priority [, policy])\n\
Set Priority.";

PyObject *
vms_sys_setpri (PyObject *self, PyObject *args)
{
	long			  l_pidadr;
	long			* al_pidadr;
	PyObject		* ar_pidadr;

	char			* at_prcnam;
	struct dsc$descriptor_s   r_prcnam;
	struct dsc$descriptor_s	* ar_prcnam;
	long			  l_prcnam_len;

	long			  l_pri;
	long			  l_prvpri;

	long			  l_policy;
	long			* al_policy;
	PyObject		* ar_policy;

	long			  l_prvpol;
	long			* al_prvpol;

	long			  l_status;

	at_prcnam = NULL;	/* should be set if 'None' was given, anyway */
	ar_policy = NULL;
	/* argument 3 (policy) is optional and only implemented on Alpha */
	if (!PyArg_ParseTuple(args, "Oz#i|O",
		&ar_pidadr, &at_prcnam, &l_prcnam_len,
		&l_pri, &ar_policy))
	{
	    return NULL;
	}

	/* argument 1 processing */
	if (ar_pidadr == Py_None)	/* argument is 'None' */
	{
	    l_pidadr  = 0;		/* skip arg 1 of SYS$SETPRI */
	    al_pidadr = &l_pidadr;	/*   but return PID	    */
	}
	else
	{
	    if (!PyInt_Check(ar_pidadr))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 1: pidadr must be integer or None");
		return NULL;
	    }
	    else
	    {
		l_pidadr  = PyInt_AsLong(ar_pidadr);
		al_pidadr = &l_pidadr;
	    }
	}

	/* argument 2 processing */
	if (at_prcnam == NULL)	/* argument is 'None' */
	{
	    ar_prcnam = 0;	/* skip arg 2 of SYS$SETPRI */
	}
	else
	{
	    /* set up descriptor */
	    r_prcnam.dsc$w_length  = (short)l_prcnam_len;
	    r_prcnam.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_prcnam.dsc$b_class   = DSC$K_CLASS_S;
	    r_prcnam.dsc$a_pointer = at_prcnam;
	    /* passed by reference */
	    ar_prcnam = &r_prcnam;
	}

	/* argument 3 handling */
#ifndef __ALPHA
	/* these arguments are not supported on VAX */
	al_policy = 0;
	al_prvpol = 0;
	l_prvpol  = 0;			/* fill in some value for return */
#else
	if (ar_policy == Py_None)	/* argument is 'None' */
	{
	    al_policy = 0; /* skip arg */
	}
	else
	{
	    if (!PyInt_Check(ar_policy))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 3: policy must be integer or None");
		return NULL;
	    }
	    else
	    {
		l_policy  = PyInt_AsLong(ar_policy);
		al_policy = &l_policy;
	    }
	}

	al_prvpol = &l_prvpol;	/* always returned */
#endif

	/* call system service */
	l_status = sys$setpri
		(al_pidadr		/* [pidadr]  */
		,ar_prcnam		/* [prcnam]  */
		,l_pri			/* pri       */
		,&l_prvpri		/* [prvpri]  */
		,al_policy		/* VAX:[nullarg], Alpha:[policy] */
		,al_prvpol		/* VAX:[nullarg], Alpha:[prvpol] */
		);
	if (l_status == SS$_NORMAL)
	{
	    /* always return 'prvpol',even on VAX where it is not implemented */
	    return Py_BuildValue ("iii", l_pidadr, l_prvpri, l_prvpol);
	}

	/* failed - error */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_setpri () */

/* ------------------------------------------------------------------------- */
static char vms_sys_setprn__doc[] =
"vms_sys.setprn ([prcnam])\n\
Set Process Name.";

PyObject *
vms_sys_setprn (PyObject *self, PyObject *args)
{
	char			* at_prcnam;
	struct dsc$descriptor_s   r_prcnam;
	struct dsc$descriptor_s	* ar_prcnam;
	long			  l_prcnam_len;

	long			  l_status;

	at_prcnam = NULL;
	if (!PyArg_ParseTuple(args, "|z#", &at_prcnam, &l_prcnam_len))
	{
	    return NULL;
	}

	if (at_prcnam == NULL)	/* argument is 'None' */
	{
	    ar_prcnam = 0;	/* skip arg 2 of SYS$SETPRI */
	}
	else
	{
	    /* set up descriptor */
	    r_prcnam.dsc$w_length  = (short)l_prcnam_len;
	    r_prcnam.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_prcnam.dsc$b_class   = DSC$K_CLASS_S;
	    r_prcnam.dsc$a_pointer = at_prcnam;
	    /* passed by reference */
	    ar_prcnam = &r_prcnam;
	}

	/* call system service */
	l_status = sys$setprn (ar_prcnam);
	if (l_status == SS$_NORMAL)
	{
	    Py_INCREF(Py_None);
	    return Py_None;
	}

	/* failed - error */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_setprn () */

/* ------------------------------------------------------------------------- */
static char vms_sys_setswm__doc[] =
"oldflg = vms_sys.setswm ([swpflg])\n\
Set Process Swap Mode.";

PyObject *
vms_sys_setswm (PyObject *self, PyObject *args)
{
	long		  l_swpflg;
	PyObject	* ar_swpflg;

	long		  l_status;

	if (!PyArg_ParseTuple(args, "O", &ar_swpflg))
	{
	    return NULL;
	}

	/* argument 1 processing */
	if (ar_swpflg == Py_None)	/* argument is 'None' */
	{
	    l_swpflg = 1;		/* disable swapping */
	}
	else
	{
	    if (!PyInt_Check(ar_swpflg))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 1: swpflg must be integer or None");
		return NULL;
	    }
	    else
	    {
		l_swpflg = PyInt_AsLong(ar_swpflg);
	    }
	}

	/* call system service */
	l_status = sys$setswm(l_swpflg);
	if ((l_status == SS$_WASCLR) || (l_status == SS$_WASSET))
	{
	    l_swpflg = (l_status == SS$_WASCLR) ? 0 : 1;
	    return Py_BuildValue ("i", l_swpflg);
	}

	/* failed - error */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_setswm () */

/* ------------------------------------------------------------------------- */
static char vms_sys_subsystem__doc[] =
"oldflg = vms_sys.subsystem ([enbflg])\n\
Subsystem.";

PyObject *
vms_sys_subsystem (PyObject *self, PyObject *args)
{
	long		  l_enbflg;
	PyObject	* ar_enbflg;

	long		  l_status;

	if (!PyArg_ParseTuple(args, "O", &ar_enbflg))
	{
	    return NULL;
	}

	/* argument 1 processing */
	if (ar_enbflg == Py_None)	/* argument is 'None' */
	{
	    l_enbflg = 0;		/* save (disable) subsystem */
	}
	else
	{
	    if (!PyInt_Check(ar_enbflg))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 1: enbflg must be integer or None");
		return NULL;
	    }
	    else
	    {
		l_enbflg = PyInt_AsLong(ar_enbflg);
	    }
	}

	/* call system service */
	l_status = sys$subsystem (l_enbflg);
	if ((l_status == SS$_WASCLR) || (l_status == SS$_WASSET))
	{
	    if (l_status == SS$_WASCLR)
	    {
		l_enbflg = 0;
	    }
	    else
	    {
		l_enbflg = 1;
	    }
	    return Py_BuildValue ("i", l_enbflg);
	}

	/* failed, error (according to the documentation this can't happen) */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_subsystem () */

/* ------------------------------------------------------------------------- */
static char vms_sys_suspnd__doc[] =
"targpid = vms_sys.suspnd ([pidadr] [,prcnam] [,flags])\n\
Suspend Process.";

PyObject *
vms_sys_suspnd (PyObject *self, PyObject *args)
{
	long			  l_pidadr;
	long			* al_pidadr;
	PyObject		* ar_pidadr;

	char			* at_prcnam;
	struct dsc$descriptor_s   r_prcnam;
	struct dsc$descriptor_s	* ar_prcnam;
	long			  l_prcnam_len;

	long			  l_flags;

	long			  l_status;

	l_flags  = 0;		/* pre-set - 0 is used even if omitted */
	/* argument 1 can be empty, 'None' or int */
	if (PyArg_ParseTuple(args, ""))
	{
	    l_pidadr  = 0;		/* skip arg 1 of SYS$SUSPND */
	    al_pidadr = &l_pidadr;	/*   but return PID	    */
	    ar_prcnam = 0;		/* skip arg 2 of SYS$SUSPND */
	}
	else
	{
	  PyErr_Clear();
	  at_prcnam = NULL; /* set defaults for optional argument */
	  /* allow int or None --------v , argument 2 is opt, none or string */
	  if (!PyArg_ParseTuple(args, "O|z#i",  /* arg 3 is optional or int */
		   &ar_pidadr, &at_prcnam, &l_prcnam_len, &l_flags))
	  {
	    return NULL;
	  }
	  else
	  {
	    /* argument 1 processing */
	    if (ar_pidadr == Py_None)	/* argument is 'None' */
	    {
	      /* turn 'None' into '0' because argument 2 might be present */
	      /*   and this routine has to return a PID */
	      l_pidadr  = 0;
	      al_pidadr = &l_pidadr;
	    }
	    else
	    {
	      if (!PyInt_Check(ar_pidadr))
	      {
		PyErr_SetString(PyExc_TypeError,
		    "argument 1: pidadr must be integer or None");
		return NULL;
	      }
	      else
	      {
		l_pidadr  = PyInt_AsLong(ar_pidadr);
		al_pidadr = &l_pidadr;
	      }
	    }

	    /* argument 2 processing */
	    if (at_prcnam == NULL)	/* argument is 'None' */
	    {
	      ar_prcnam = 0;	/* skip arg 2 of SYS$SUSPND */
	    }
	    else
	    {
	      /* set up descriptor */
	      r_prcnam.dsc$w_length  = (short)l_prcnam_len;
	      r_prcnam.dsc$b_dtype   = DSC$K_DTYPE_T;
	      r_prcnam.dsc$b_class   = DSC$K_CLASS_S;
	      r_prcnam.dsc$a_pointer = at_prcnam;
	      /* passed by reference */
	      ar_prcnam = &r_prcnam;
	    }
	  }
	}

	/* call system service */
	l_status = sys$suspnd
		(al_pidadr		/* [pidadr] */
		,ar_prcnam		/* [prcnam] */
		,l_flags		/* flags    */
		);
	if (l_status == SS$_NORMAL)
	{
	    /* return PID of target process */
	    return Py_BuildValue ("l", l_pidadr);
	}

	/* call failed - error */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_suspnd () */

/* ------------------------------------------------------------------------- */
static char vms_sys_timcon__doc[] =
"utcadr = vms_sys.timcon ([timadr] , cvtflg=1)\n\
timadr = vms_sys.timcon ([utcadr] , cvtflg=0)\n\
Time Converter.";

PyObject *
vms_sys_timcon (PyObject *self, PyObject *args)
{
	long			  q_timadr[2];	/* binary system time */
	long			  o_utcadr[4];	/* utc_date_time      */
	long			* ax_argtim;

	char			  t_timbuf[32];	/* hex string 16/32 char */
	/* ----- */

	PyObject		* ar_argtim;	/* argument 1	   */
	long			  l_argtim_len;	/* length of arg 1 */

	long			  l_cvtflg;	/* conversion flag $TIMCON */

	long			  l_status;

	/* argument 1 can be empty, 'None' or string/PyLongInt */
	l_cvtflg     = 0;			/* default */
	l_argtim_len = 0;
	/* allow str,PyLongInt,None -vv-- argument 2 must be integer */
	if (!PyArg_ParseTuple(args, "Ol", &ar_argtim, &l_cvtflg))
	{
	    return NULL;
	}

	/* argument is 'None' or "" */
	if ((ar_argtim == NULL) || (ar_argtim == Py_None))
	{
	      /* no argument given, ommit argument */
	      /* to comply with SYS$TIMCON */
	      ax_argtim = 0;
	}
	else
	{
	  /* l_cvtflg = 0 -- convert 128bit utc to 64bit bintim */
	  /*	ar_argtim must be *exactly* 32 (hex) characters! */
	  if (l_cvtflg == 0)
	  {
	    if (!PyString_Check(ar_argtim))
	    {	  
		PyErr_SetString(PyExc_ValueError,
		    "argument 1: must be 32 hex characters");
		return NULL;
	    }
	    else
	    {
	      if (PyString_Size(ar_argtim) != 32)
	      {
		PyErr_SetString(PyExc_ValueError,
		    "argument 1: must be 32 hex characters");
		return NULL;
	      }
	      else
	      {
		/* convert hex string to binary octaword */
		l_status = vms__cvt_hex2oct
			(PyString_AS_STRING (ar_argtim)
			,&o_utcadr[0]
			);
		if (l_status != SS$_NORMAL)
		{
		  PyErr_SetString(PyExc_ValueError,
		    "argument 1: must be 32 hex characters");
		  return NULL;
		}
		ax_argtim = &o_utcadr[0];
	      } /* string size == 32 ?? */
	    } /* (!PyString_Check(ar_argtim)) */
	  } /* (l_cvtflg == 0) */
	  else
	  /* l_cvtflg = 1 -- convert 64bit bintim to 128bit utc */
	  /*	ar_argtim must be Python Long Integer ! */
	  {
	    if (!PyLong_Check(ar_argtim))
	    {
	      PyErr_SetString(PyExc_ValueError,
		"argument 1: must be long int");
	      return NULL;
	    }
	    /* convert PyLongInt to binary quadword */
	    l_status = vms__cvt_pylong2quad (ar_argtim, &q_timadr[0]);
	    if (l_status == -1)
	    {
	      /* error string has been set */
	      return NULL;
	    }
	    ax_argtim = &q_timadr[0];
	  } /* if (l_cvtflg == 0) */
	} /* if ((ar_argtim == NULL) || (ar_argtim == Py_None)) */

	/* ---------- */

	/* l_cvtflg = 0 -- convert 128bit utc to 64bit bintim */
	/*	ar_argtim must be *exactly* 32 (hex) characters! */
	if (l_cvtflg == 0)
	{
	    l_status = sys$timcon
		(&q_timadr[0]		/* [timadr] */
		,ax_argtim		/* [utcadr] */
		,l_cvtflg		/* cvtflg   */
		);
	    if (l_status == SS$_NORMAL)
	    {
		/* convert binary quadword to PyLongInt */
		return vms__cvt_quad2pylong (&q_timadr[0]);
	    }
	}
	else
	{
	    l_status = sys$timcon
		(ax_argtim		/* [timadr] */
		,&o_utcadr[0]		/* [utcadr] */
		,l_cvtflg		/* cvtflg   */
		);
	    if (l_status == SS$_NORMAL)
	    {
		/* convert binary octaword to hex string to return to Python */
		l_status = vms__cvt_oct2hex (&o_utcadr[0], &t_timbuf[0]);
		if (l_status == SS$_NORMAL)
		{
		    return Py_BuildValue ("s#", &t_timbuf[0], (int)32);
		}
		/* conversion failed - fall through to error */
	    }
	} /* if (l_cvtflg == 0) */

	/* failed */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_timcon () */

/* ------------------------------------------------------------------------- */
static char vms_sys_wake__doc[] =
"targpid = vms_sys.wake ([pidadr] [,prcnam])\n\
Wake Process from Hibernation.";

PyObject *
vms_sys_wake (PyObject *self, PyObject *args)
{
	long			  l_pidadr;
	long			* al_pidadr;
	PyObject		* ar_pidadr;

	char			* at_prcnam;
	struct dsc$descriptor_s   r_prcnam;
	struct dsc$descriptor_s * ar_prcnam;
	long			  l_prcnam_len;

	long			  l_status;

	/* argument 1 can be empty, 'None' or int */
	if (PyArg_ParseTuple(args, ""))
	{
	    l_pidadr  = 0;		/* skip arg 1 of SYS$WAKE */
	    al_pidadr = &l_pidadr;	/*   but return PID	  */
	    ar_prcnam = 0;		/* skip arg 2 of SYS$WAKE */
	}
	else
	{
	  PyErr_Clear();
	  at_prcnam = NULL; /* set defaults for optional argument */
	  /* allow int or None --------v , argument 2 is opt, none or string */
	  if (!PyArg_ParseTuple(args, "O|z#",
		   &ar_pidadr, &at_prcnam, &l_prcnam_len))
	  {
	    return NULL;
	  }
	  else
	  {
	    /* argument 1 processing */
	    if (ar_pidadr == Py_None)	/* argument is 'None' */
	    {
	      /* turn 'None' into '0' because argument 2 might be present */
	      /*   and this routine has to return a PID */
	      l_pidadr  = 0;
	      al_pidadr = &l_pidadr;
	    }
	    else
	    {
	      if (!PyInt_Check(ar_pidadr))
	      {
		PyErr_SetString(PyExc_TypeError,
		    "argument 1: pidadr must be integer or None");
		return NULL;
	      }
	      else
	      {
		l_pidadr  = PyInt_AsLong(ar_pidadr);
		al_pidadr = &l_pidadr;
	      }
	    }

	    /* argument 2 processing */
	    if (at_prcnam == NULL)	/* argument is 'None' */
	    {
	      ar_prcnam = 0;		/* skip arg 2 of SYS$CANWAK */
	    }
	    else
	    {
	      /* set up descriptor */
	      r_prcnam.dsc$w_length  = (short)l_prcnam_len;
	      r_prcnam.dsc$b_dtype   = DSC$K_DTYPE_T;
	      r_prcnam.dsc$b_class   = DSC$K_CLASS_S;
	      r_prcnam.dsc$a_pointer = at_prcnam;
	      /* passed by reference */
	      ar_prcnam = &r_prcnam;
	    }
	  }
	}

	/* call system service */
	l_status = sys$wake (al_pidadr, ar_prcnam);
	if (l_status == SS$_NORMAL)
	{
	    /* return PID of target process */
	    return Py_BuildValue ("l", l_pidadr);
	}

	/* failed - error */
	PyVMS_ErrSetVal(vms_sys_Error, 1, l_status);
	return NULL;
} /* vms_sys_wake () */

/* ------------------------------------------------------------------------- */
static struct PyMethodDef vms_sys_methods[] = {
	{"asctim",	(PyCFunction)vms_sys_asctim,	METH_VARARGS, vms_sys_asctim__doc},
	{"asctoid",	(PyCFunction)vms_sys_asctoid,	METH_VARARGS, vms_sys_asctoid__doc},
/*V60*/	{"ascutc",	(PyCFunction)vms_sys_ascutc,	METH_VARARGS, vms_sys_ascutc__doc}, 
	{"bintim",	(PyCFunction)vms_sys_bintim,	METH_VARARGS, vms_sys_bintim__doc},
/*V60*/	{"binutc",	(PyCFunction)vms_sys_binutc,	METH_VARARGS, vms_sys_binutc__doc},
	{"canwak",	(PyCFunction)vms_sys_canwak,	METH_VARARGS, vms_sys_canwak__doc},
	{"delprc",	(PyCFunction)vms_sys_delprc,	METH_VARARGS, vms_sys_delprc__doc},
	{"forcex",	(PyCFunction)vms_sys_forcex,	METH_VARARGS, vms_sys_forcex__doc},
	{"gettim",	(PyCFunction)vms_sys_gettim,	METH_VARARGS, vms_sys_gettim__doc},
/*V60*/	{"getutc",	(PyCFunction)vms_sys_getutc,	METH_VARARGS, vms_sys_getutc__doc},
	{"hiber",	(PyCFunction)vms_sys_hiber,	METH_VARARGS, vms_sys_hiber__doc},
	{"numtim",	(PyCFunction)vms_sys_numtim,	METH_VARARGS, vms_sys_numtim__doc},
	{"numutc",	(PyCFunction)vms_sys_numutc,	METH_VARARGS, vms_sys_numutc__doc},
	{"purgws",	(PyCFunction)vms_sys_purgws,	METH_VARARGS, vms_sys_purgws__doc},
	{"resume",	(PyCFunction)vms_sys_resume,	METH_VARARGS, vms_sys_resume__doc},
	{"schdwk",	(PyCFunction)vms_sys_schdwk,	METH_VARARGS, vms_sys_schdwk__doc},
	{"setddir",	(PyCFunction)vms_sys_setddir,	METH_VARARGS, vms_sys_setddir__doc},
	{"setpri",	(PyCFunction)vms_sys_setpri,	METH_VARARGS, vms_sys_setpri__doc},
	{"setprn",	(PyCFunction)vms_sys_setprn,	METH_VARARGS, vms_sys_setprn__doc},
	{"setswm",	(PyCFunction)vms_sys_setswm,	METH_VARARGS, vms_sys_setswm__doc},
/*V60*/	{"subsystem",	(PyCFunction)vms_sys_subsystem,	METH_VARARGS, vms_sys_subsystem__doc},
	{"suspnd",	(PyCFunction)vms_sys_suspnd,	METH_VARARGS, vms_sys_suspnd__doc},
/*V60*/	{"timcon",	(PyCFunction)vms_sys_timcon,	METH_VARARGS, vms_sys_timcon__doc},
	{"wake",	(PyCFunction)vms_sys_wake,	METH_VARARGS, vms_sys_wake__doc},
	{NULL,		NULL}
};

void
initvms_sys()
{
	PyObject *m, *d;

	m = Py_InitModule4("vms_sys",
			   vms_sys_methods,
			   vms_sys_doc__,
			   (PyObject *)NULL,
			   PYTHON_API_VERSION);
	d = PyModule_GetDict(m);

	/* ----- ----- ----- ----- */
	/* Initialize exception */
	vms_sys_Error = PyErr_NewException("vms_sys.error", NULL, NULL);
	if (vms_sys_Error == NULL || PyDict_SetItemString(d, "error", vms_sys_Error) != 0)
	    Py_FatalError("can't define vms_sys.error");

	/* ----- ----- ----- ----- */
	/* Check for errors */
        if (PyErr_Occurred())
                Py_FatalError("can't initialize module vms_sys");
	/* ----- ----- ----- ----- */
} /* initvms_sys () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SYS.C */
